@@ -931,7 +931,7 @@ class Mail_RFC822 {
      * @access private
      */
     function raiseError($message) {
-        debugLog("z_RFC822 error: ". $message);
+        ZLog::Write(LOGLEVEL_ERROR, "z_RFC822 error: ". $message);
         return false;
     }
 }
\ No newline at end of file
