@@ -11,7 +11,7 @@
 *
 * Created   :   20.10.2011
 *
-* Copyright 2007 - 2011 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -64,7 +64,7 @@ class LoopDetection extends InterProcessData {
      */
     public function LoopDetection() {
         // initialize super parameters
-        $this->allocate = 204800; // 200 KB
+        $this->allocate = 1024000; // 1 MB
         $this->type = 1337;
         parent::__construct();
 
@@ -178,6 +178,20 @@ class LoopDetection extends InterProcessData {
         return true;
     }
 
+    /**
+     * Marks the current process as a PUSH connection
+     *
+     * @access public
+     * @return boolean
+     */
+    public function ProcessLoopDetectionSetAsPush() {
+        // generate entry if not already there
+        self::GetProcessEntry();
+        self::$processentry['push'] = true;
+
+        return $this->updateProcessStack();
+    }
+
     /**
      * Indicates if a full Hierarchy Resync is necessary
      *
@@ -255,7 +269,7 @@ class LoopDetection extends InterProcessData {
         $stack = $this->getProcessStack();
         if (count($stack) > 1) {
             $se = $stack[0];
-            if (!isset($se['end']) && $se['cc'] != ZPush::COMMAND_PING) {
+            if (!isset($se['end']) && $se['cc'] != ZPush::COMMAND_PING && !isset($se['push']) ) {
                 // there is no end time
                 ZLog::Write(LOGLEVEL_ERROR, sprintf("LoopDetection->ProcessLoopDetectionPreviousConnectionFailed(): Command '%s' at %s with pid '%d' terminated unexpectedly or is still running.", Utils::GetCommandFromCode($se['cc']), Utils::GetFormattedTime($se['time']), $se['pid']));
                 ZLog::Write(LOGLEVEL_ERROR, "Please check your logs for this PID and errors like PHP-Fatals or Apache segmentation faults and report your results to the Z-Push dev team.");
@@ -476,6 +490,44 @@ class LoopDetection extends InterProcessData {
         return $okIds;
     }
 
+    /**
+     * Marks a SyncState as "already used", e.g. when an import process started.
+     * This is most critical for DiffBackends, as an imported message would be exported again
+     * in the heartbeat if the notification is triggered before the import is complete.
+     *
+     * @param string $folderid          folder id
+     * @param string $uuid              synkkey
+     * @param string $counter           synckey counter
+     *
+     * @access public
+     * @return boolean
+     */
+    public function SetSyncStateUsage($folderid, $uuid, $counter) {
+        // initialize params
+        $this->InitializeParams();
+
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->SetSyncStateUsage(): uuid: %s  counter: %d", $uuid, $counter));
+
+        // exclusive block
+        if ($this->blockMutex()) {
+            $loopdata = ($this->hasData()) ? $this->getData() : array();
+            // check and initialize the array structure
+            $this->checkArrayStructure($loopdata, $folderid);
+            $current = $loopdata[self::$devid][self::$user][$folderid];
+
+
+            // update the usage flag
+            $current["usage"] = $counter;
+
+            // update loop data
+            $loopdata[self::$devid][self::$user][$folderid] = $current;
+            $ok = $this->setData($loopdata);
+
+            $this->releaseMutex();
+        }
+        // end exclusive block
+    }
+
     /**
      * Checks if the given counter for a certain uuid+folderid was exported before.
      * Returns also true if the counter are the same but previously there were
@@ -506,18 +558,20 @@ class LoopDetection extends InterProcessData {
             $current = $loopdata[self::$devid][self::$user][$folderid];
 
             if (!empty($current)) {
-                if ($current["uuid"] != $uuid) {
-                    ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, uuid changed");
+                if (!isset($current["uuid"]) || $current["uuid"] != $uuid) {
+                    ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, uuid changed or not set");
                     $obsolete = true;
                 }
-                ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->IsSyncStateObsolete(): check uuid counter: %d - last known counter: %d with %d queued objects", $counter, $current["count"], $current["queued"]));
+                else {
+                    ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->IsSyncStateObsolete(): check uuid counter: %d - last known counter: %d with %d queued objects", $counter, $current["count"], $current["queued"]));
 
-                if ($current["uuid"] == $uuid && ($current["count"] > $counter || ($current["count"] == $counter && $current["queued"] > 0))) {
-                    ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, counter already processed");
-                    $obsolete = true;
+                    if ($current["uuid"] == $uuid && ($current["count"] > $counter || ($current["count"] == $counter && $current["queued"] > 0) || (isset($current["usage"]) && $current["usage"] >= $counter))) {
+                        $usage = isset($current["usage"]) ? sprintf(" - counter %d already expired",$current["usage"]) : "";
+                        ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->IsSyncStateObsolete(): yes, counter already processed". $usage);
+                        $obsolete = true;
+                    }
                 }
             }
-
         }
 
         return $obsolete;
@@ -618,6 +672,8 @@ class LoopDetection extends InterProcessData {
                     // case 1.1
                     $current['count'] = $counter;
                     $current['queued'] = $queuedMessages;
+                    if (isset($current["usage"]) && $current["usage"] < $current['count'])
+                        unset($current["usage"]);
 
                     // case 1.2
                     if (isset($current['maxCount'])) {
@@ -643,6 +699,8 @@ class LoopDetection extends InterProcessData {
                 // case 2 - same counter, but there were no changes before and are there now
                 else if ($current['count'] == $counter && $current['queued'] == 0 && $queuedMessages > 0) {
                     $current['queued'] = $queuedMessages;
+                    if (isset($current["usage"]) && $current["usage"] < $current['count'])
+                        unset($current["usage"]);
                 }
 
                 // case 3 - same counter, changes sent before, hanging loop and ignoring
