@@ -6,7 +6,7 @@
 *
 * Created   :   03.04.2008
 *
-* Copyright 2007 - 2011 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -134,6 +134,110 @@ class Utils {
         return ($out)?$out:null;
     }
 
+    /**
+     * Build the fileas string from the components according to the configuration.
+     *
+     * @param string $lastname
+     * @param string $firstname
+     * @param string $middlename
+     * @param string $company
+     *
+     * @access public
+     * @return string fileas
+     */
+    static public function BuildFileAs($lastname = "", $firstname = "", $middlename = "", $company = "") {
+        if (defined('FILEAS_ORDER')) {
+            $fileas = $lastfirst = $firstlast = "";
+            $names = trim ($firstname . " " . $middlename);
+            $lastname = trim($lastname);
+            $company = trim($company);
+
+            // lastfirst is "lastname, firstname middlename"
+            // firstlast is "firstname middlename lastname"
+            if (strlen($lastname) > 0) {
+                $lastfirst = $lastname;
+                if (strlen($names) > 0){
+                    $lastfirst .= ", $names";
+                    $firstlast = "$names $lastname";
+                }
+                else {
+                    $firstlast = $lastname;
+                }
+            }
+            elseif (strlen($names) > 0) {
+                $lastfirst = $firstlast = $names;
+            }
+
+            // if fileas with a company is selected
+            // but company is emtpy then it will
+            // fallback to firstlast or lastfirst
+            // (depending on which is selected for company)
+            switch (FILEAS_ORDER) {
+                case SYNC_FILEAS_COMPANYONLY:
+                    if (strlen($company) > 0) {
+                        $fileas = $company;
+                    }
+                    elseif (strlen($firstlast) > 0)
+                        $fileas = $lastfirst;
+                    break;
+                case SYNC_FILEAS_COMPANYLAST:
+                    if (strlen($company) > 0) {
+                        $fileas = $company;
+                        if (strlen($lastfirst) > 0)
+                            $fileas .= "($lastfirst)";
+                    }
+                    elseif (strlen($lastfirst) > 0)
+                        $fileas = $lastfirst;
+                    break;
+                case SYNC_FILEAS_COMPANYFIRST:
+                    if (strlen($company) > 0) {
+                        $fileas = $company;
+                        if (strlen($firstlast) > 0) {
+                            $fileas .= " ($firstlast)";
+                        }
+                    }
+                    elseif (strlen($firstlast) > 0) {
+                        $fileas = $firstlast;
+                    }
+                    break;
+                case SYNC_FILEAS_FIRSTCOMPANY:
+                    if (strlen($firstlast) > 0) {
+                        $fileas = $firstlast;
+                        if (strlen($company) > 0) {
+                            $fileas .= " ($company)";
+                        }
+                    }
+                    elseif (strlen($company) > 0) {
+                        $fileas = $company;
+                    }
+                    break;
+                case SYNC_FILEAS_LASTCOMPANY:
+                    if (strlen($lastfirst) > 0) {
+                        $fileas = $lastfirst;
+                        if (strlen($company) > 0) {
+                            $fileas .= " ($company)";
+                        }
+                    }
+                    elseif (strlen($company) > 0) {
+                        $fileas = $company;
+                    }
+                    break;
+                case SYNC_FILEAS_LASTFIRST:
+                    if (strlen($lastfirst) > 0) {
+                        $fileas = $lastfirst;
+                    }
+                    break;
+                default:
+                    $fileas = $firstlast;
+                    break;
+            }
+            if (strlen($fileas) == 0)
+                ZLog::Write(LOGLEVEL_DEBUG, "Fileas is empty.");
+            return $fileas;
+        }
+        ZLog::Write(LOGLEVEL_DEBUG, "FILEAS_ORDER not defined. Add it to your config.php.");
+        return null;
+    }
     /**
      * Checks if the PHP-MAPI extension is available and in a requested version
      *
@@ -316,6 +420,9 @@ class Utils {
      * @return string truncated string
      */
     static public function Utf8_truncate($string, $length) {
+        // make sure length is always an interger
+        $length = (int)$length;
+
         if (strlen($string) <= $length)
             return $string;
 
@@ -635,6 +742,150 @@ class Utils {
         else
             return @strftime("%d/%m/%Y %H:%M:%S", $timestamp);
     }
+
+
+   /**
+    * Get charset name from a codepage
+    *
+    * @see http://msdn.microsoft.com/en-us/library/dd317756(VS.85).aspx
+    *
+    * Table taken from common/codepage.cpp
+    *
+    * @param integer codepage Codepage
+    *
+    * @access public
+    * @return string iconv-compatible charset name
+    */
+    public static function GetCodepageCharset($codepage) {
+        $codepages = array(
+            20106 => "DIN_66003",
+            20108 => "NS_4551-1",
+            20107 => "SEN_850200_B",
+            950 => "big5",
+            50221 => "csISO2022JP",
+            51932 => "euc-jp",
+            51936 => "euc-cn",
+            51949 => "euc-kr",
+            949 => "euc-kr",
+            936 => "gb18030",
+            52936 => "csgb2312",
+            852 => "ibm852",
+            866 => "ibm866",
+            50220 => "iso-2022-jp",
+            50222 => "iso-2022-jp",
+            50225 => "iso-2022-kr",
+            1252 => "windows-1252",
+            28591 => "iso-8859-1",
+            28592 => "iso-8859-2",
+            28593 => "iso-8859-3",
+            28594 => "iso-8859-4",
+            28595 => "iso-8859-5",
+            28596 => "iso-8859-6",
+            28597 => "iso-8859-7",
+            28598 => "iso-8859-8",
+            28599 => "iso-8859-9",
+            28603 => "iso-8859-13",
+            28605 => "iso-8859-15",
+            20866 => "koi8-r",
+            21866 => "koi8-u",
+            932 => "shift-jis",
+            1200 => "unicode",
+            1201 => "unicodebig",
+            65000 => "utf-7",
+            65001 => "utf-8",
+            1250 => "windows-1250",
+            1251 => "windows-1251",
+            1253 => "windows-1253",
+            1254 => "windows-1254",
+            1255 => "windows-1255",
+            1256 => "windows-1256",
+            1257 => "windows-1257",
+            1258 => "windows-1258",
+            874 => "windows-874",
+            20127 => "us-ascii"
+        );
+
+        if(isset($codepages[$codepage])) {
+            return $codepages[$codepage];
+        } else {
+            // Defaulting to iso-8859-15 since it is more likely for someone to make a mistake in the codepage
+            // when using west-european charsets then when using other charsets since utf-8 is binary compatible
+            // with the bottom 7 bits of west-european
+            return "iso-8859-15";
+        }
+    }
+
+    /**
+     * Converts a string encoded with codepage into an UTF-8 string
+     *
+     * @param int $codepage
+     * @param string $string
+     *
+     * @access public
+     * @return string
+     */
+    public static function ConvertCodepageStringToUtf8($codepage, $string) {
+        if (function_exists("iconv")) {
+            $charset = self::GetCodepageCharset($codepage);
+
+            return iconv($charset, "utf-8", $string);
+        }
+        return $string;
+    }
+
+    /**
+     * Returns the best match of preferred body preference types.
+     *
+     * @param array             $bpTypes
+     *
+     * @access public
+     * @return int
+     */
+    public static function GetBodyPreferenceBestMatch($bpTypes) {
+        // The best choice is RTF, then HTML and then MIME in order to save bandwidth
+        // because MIME is a complete message including the headers and attachments
+        if (in_array(SYNC_BODYPREFERENCE_RTF, $bpTypes))  return SYNC_BODYPREFERENCE_RTF;
+        if (in_array(SYNC_BODYPREFERENCE_HTML, $bpTypes)) return SYNC_BODYPREFERENCE_HTML;
+        if (in_array(SYNC_BODYPREFERENCE_MIME, $bpTypes)) return SYNC_BODYPREFERENCE_MIME;
+        return SYNC_BODYPREFERENCE_PLAIN;
+    }
+
+    /* BEGIN fmbiete's contribution r1516, ZP-318 */
+    /**
+     * Converts a html string into a plain text string
+     *
+     * @param string $html
+     *
+     * @access public
+     * @return string
+     */
+    public static function ConvertHtmlToText($html) {
+        // remove css-style tags
+        $plaintext = preg_replace("/<style.*?<\/style>/is", "", $html);
+        // remove all other html
+        $plaintext = strip_tags($plaintext);
+
+        return $plaintext;
+    }
+    /* END fmbiete's contribution r1516, ZP-318 */
+
+    /**
+     * Returns AS-style LastVerbExecuted value from the server value.
+     *
+     * @param int $verb
+     *
+     * @access public
+     * @return int
+     */
+    public static function GetLastVerbExecuted($verb) {
+        switch ($verb) {
+            case NOTEIVERB_REPLYTOSENDER:   return AS_REPLYTOSENDER;
+            case NOTEIVERB_REPLYTOALL:      return AS_REPLYTOALL;
+            case NOTEIVERB_FORWARD:         return AS_FORWARD;
+        }
+
+        return 0;
+    }
 }
 
 
